/* global describe, it, beforeEach  */

const assert = require('assert')
const jsdom = require('jsdom')

;(() => {
  describe('JSMiLe-bundler', () => {
    let window

    beforeEach(() => {
      const virtualConsole = new jsdom.VirtualConsole()
      virtualConsole.sendTo(console)

      return jsdom
        .JSDOM
        .fromFile(
          'test/test.html',
          {
            resources: 'usable',
            runScripts: 'dangerously',
            virtualConsole
          }
        )
        .then((dom) => { window = dom.window })
    })

    describe('render', () => {
      it('is not automatically available in node', () => {
        assert.strictEqual(global.jsml, undefined)
      })

      it('automatically provides render', done => {
        window.addEventListener('load', () => {
          const greeting = window.document.querySelector('div').textContent
          assert.strictEqual(greeting, 'hello world')

          done()
        })
      })

      it('automatically provides jsml', done => {
        window.addEventListener('load', () => {
          const exclamation = window.document.querySelector('span').textContent
          assert.strictEqual(exclamation, '!')

          done()
        })
      })

      it('automatically provides render', done => {
        window.addEventListener('load', () => {
          const greeting = window.document.querySelector('div').textContent
          assert.strictEqual(greeting, 'hello world')

          done()
        })
      })

      it('appends to all elements in a jQuery collection', done => {
        window.addEventListener('load', () => {
          const items = window.document.querySelectorAll('li')

          for (const item of items) {
            assert.strictEqual(item.textContent, 'something!')
          }

          done()
        })
      })

      it('appends all elements in a jQuery collection', done => {
        window.addEventListener('load', () => {
          const list = window.document.querySelector('ul')

          assert.strictEqual(list.childNodes.length, 3)

          list.childNodes.forEach(node => {
            assert.strictEqual(node.tagName, 'LI')
            assert.strictEqual(node.textContent, 'something!')
          })

          done()
        })
      })
    })

    describe('views', () => {
      it('exposes views', done => {
        window.addEventListener('load', () => {
          assert(window.jsmile.views.title)

          done()
        })
      })

      it('passes an empty options object by default', done => {
        window.addEventListener('load', () => {
          const content = window.document.querySelector('.content')

          const textContent = content.textContent

          assert.strictEqual(textContent, '')

          done()
        })
      })

      it('supports nested includes', done => {
        window.addEventListener('load', () => {
          const content = window.document.querySelector('.title')

          assert.strictEqual(content.textContent, 'title')

          done()
        })
      })

      it('supports recursive view lookup', done => {
        window.addEventListener('load', () => {
          const heading = window.document.querySelector('.heading')

          assert.strictEqual(heading.textContent, 'heading')

          done()
        })
      })

      it('runs matching includers', done => {
        window.addEventListener('load', () => {
          const note = window.document.querySelector('.note')

          assert.strictEqual(note.tagName, 'DIV')

          done()
        })
      })
    })

    describe('dependencies', () => {
      it('exposes includers', done => {
        window.addEventListener('load', () => {
          assert(window.jsmile.includers.need)

          done()
        })
      })

      it('passes an empty options object by default', done => {
        window.addEventListener('load', () => {
          const needer = window.document.querySelector('.needer')

          assert.strictEqual(needer.classList.value, 'needer')

          done()
        })
      })

      it('passes options to dependencies', done => {
        window.addEventListener('load', () => {
          const needing = window.document.querySelector('.needing')
          assert.strictEqual(needing.tagName, 'SPAN')

          done()
        })
      })

      it('only runs includers once', done => {
        window.addEventListener('load', () => {
          const needers = window.document.querySelectorAll('.needer')
          assert.strictEqual(needers.length, 2)

          const needings = window.document.querySelectorAll('.needing')
          assert.strictEqual(needings.length, 1)

          const notes = window.document.querySelectorAll('.note')
          assert.strictEqual(notes.length, 1)

          done()
        })
      })

      it('returns null inside of rendered content', done => {
        window.addEventListener('load', () => {
          assert.strictEqual(window.notNeeded, null)

          done()
        })
      })
    })
  })
})()
